<?php
/**
 * @version   $id: $
 * @author    JoomlaUX!
 * @package   Joomla!
 * @subpackage  mod_jux_news_ticker
 * @copyright Copyright (C) 2012 - 2014 by Joomlaux. All rights reserved.
 * @license   http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL version 3, See LICENSE.txt
 */

// no direct access
defined('_JEXEC') or die('Restricted access'); 

JModelLegacy::addIncludePath(JPATH_SITE . '/components/com_content/models', 'ContentModel');

if (!class_exists('modJUXNewsTickerCss')) {
    require_once dirname(__FILE__) . '/includes/css.php';
}

class mod_JUX_NewsTickerHelper {
    public static function getCssProcessor(&$params, $filename, $prefix) {
        return modJUXNewsTickerCss::process($params, $filename, $prefix);
    }
}

abstract class modJUXNewsTickerHelper {

	static function loadCss($params) {
        $document   = JFactory::getDocument();
        
        //Display options
		$document->addStyleSheet(JURI::base() . 'modules/mod_jux_news_ticker/assets/css/style.css');
		$document->addStyleSheet(JURI::base() . 'modules/mod_jux_news_ticker/assets/css/font-awesome.min.css');
		$ticker_style = $params->get('ticker_style');

		//Ticker type style
		if ($ticker_style == "type" || $ticker_style == "scroll") {
		    $document->addStyleSheet(JURI::base() . 'modules/mod_jux_news_ticker/assets/css/dark.css');
		}
    }

	// Enable Jquery
	static function loadJquery($params) {
        $document = JFactory::getDocument();

        Joomla\CMS\HTML\HTMLHelper::_('jquery.framework');
		$document->addScript(JURI::base() . 'modules/mod_jux_news_ticker/assets/js/jquery-migrate-1.4.1.min.js');

        $enable_noconflict = $params->get('enable_noconflict', 0);
        if ($enable_noconflict) {
            $document->addCustomTag('<script type="text/javascript">jQuery.noConflict();</script>');
        }

        $document->addScript(JURI::base() . 'modules/mod_jux_news_ticker/assets/js/jquery.stickit.js');
		$document->addScript(JURI::base() . 'modules/mod_jux_news_ticker/assets/js/acmeticker.js');

		if ($params->get('sticky') == 1) {
		    $document->addScript(JURI::base() . 'modules/mod_jux_news_ticker/assets/js/jquery.sticky.js');
		}
    }

    static function timeFormat($params){
    	$time = $params->get('time');
    	$custom_time = $params->get('custom_time');
		if($custom_time){
			$time = $custom_time;
		}
		else{
			$time = $time;	
		}
		$date_format = JText::_($time);
    	return $date_format;
    }

	public static function getList(&$params){
		$data = array();
		$display_form = strtolower($params->get('display_form','joomla_content'));
		if ($display_form == 'joomla_content'){
			 if ($params->get('enable_cache')) {
			 	$cache = JFactory::getCache();
                $cache->setCaching(true);
                $cache->setLifeTime($params->get('cache_time', 30) * 60);
                $rows = $cache->get(array((new modJUXNewsTickerHelper()), 'getListArticles'), array($params));
			 	
			 }else{
			 	$data = self::getListArticles($params);
			 }
		
		}
		return $data;
	}

	/** 
	 * Method get list external links
	 * @param array $params
	 * 
	 *  @return array
	 */
	public static function getExternalsLink(&$params){
        $external_links = $params->get('external_links');
        return $external_links;
	}
	
	/**
	 * Method get list articles
	 * @param array $params
	 * 
	 * @return array $items
	 */
	public static function getListArticles(&$params){
		// Get the dbo
		$db = JFactory::getDbo();

		// Get an instance of the generic articles model
		$model = JModelLegacy::getInstance('Articles', 'ContentModel', array('ignore_request' => true));

		// Set application parameters in model
		$app = JFactory::getApplication();
		$appParams = $app->getParams();
		$model->setState('params', $appParams);

		// Set the filters based on the module params
		$model->setState('list.start', 0);
		$model->setState('list.limit', (int) $params->get('count', 5));
		$model->setState('filter.published', 1);

		// Access filter
		$access = !JComponentHelper::getParams('com_content')->get('show_noauth');
		$authorised = JAccess::getAuthorisedViewLevels(JFactory::getUser()->get('id'));
		$model->setState('filter.access', $access);

		// Category filter
		$model->setState('filter.category_id', $params->get('catid', array()));

		// Filter by language
		$model->setState('filter.language', $app->getLanguageFilter());
		
		$ordering = $params->get('sort_order_field', 'created');
		//$dir = $params->get('sort_order', 'DESC');
		switch ($ordering)
		{

			case 'date' :
				$orderby = 'a.created';
				$dir = 'ASC';
				break;

			case 'rdate' :
				$orderby = 'a.created';
				$dir = 'DESC';
				break;

			case 'alpha' :
				$orderby = 'a.title';
				$dir = 'ASC';
				break;

			case 'ralpha' :
				$orderby = 'a.title';
				$dir = 'DESC';
				break;

			case 'order' :
				$orderby = 'a.ordering';
				$dir = 'ASC';
				break;

			case 'rorder' :
				$orderby = 'a.ordering';
				$dir = 'DESC';
				break;

			case 'hits' :
				$orderby = 'a.hits';
				$dir = 'DESC';
				break;

			case 'rand' :
				$orderby = 'RAND()';
				$dir = '';
				break;
				
			case 'modified' :
				$orderby = 'modified';
				$dir = 'DESC';
				break;

			case 'publish_up' :
				$orderby = 'a.publish_up';
				$dir = 'DESC';
				break;
				
			case 'id':
			default :
				$orderby = 'a.id';
				$dir = 'DESC';
			break;
		}
			
		$model->setState('list.ordering',$orderby);
		$model->setState('list.direction', $dir);

		$items = $model->getItems();

		foreach ($items as &$item)
		{
			$item->slug = $item->id . ':' . $item->alias;
			$item->catslug = $item->catid . ':' . $item->category_alias;

			if ($access || in_array($item->access, $authorised))
			{
				// We know that user has the privilege to view the article
				$item->link = JRoute::_(ContentHelperRoute::getArticleRoute($item->slug, $item->catslug));
			}
			else
			{
				$item->link = JRoute::_('index.php?option=com_users&view=login');
			}
		}

		return $items;
	}
	/**
	 * Method trim string with max specify
	 * @param string $string
	 * @param int $maxChar
	 * 
	 * @return string
	 */
	public static function trimChar($string,$maxChar = 50){
		
		if (strlen($string) > $maxChar)
			return JString::substr($string,0,$maxChar).' ...';
			
		return $string;
	}
 	/**
     * get parameters from configuration string.
     *
     * @param string $string;
     * @return array.
     */
    public static function parseExternalsLink($string)
    {
        $string = html_entity_decode($string, ENT_QUOTES);
        $regex = "/\s*([^=\s]+)\s*=\s*('([^']*)'|\"([^\"]*)\"|([^\s]*))/";
        $linkParams = null;
        if (preg_match_all($regex, $string, $matches)) {
            for ($i = 0; $i < count($matches[1]); $i++) {
                $key = $matches[1][$i];
                $value = $matches[3][$i] ? $matches[3][$i] : ($matches[4][$i] ? $matches[4][$i] : $matches[5][$i]);
                $linkParams[$key] = $value;
            }
        }
        return $linkParams;
    }
}